\name{lamhosts}
\alias{mpi.is.master}
\alias{lamhosts}
\alias{mpi.hostinfo}
\alias{slave.hostinfo}
\title{Hosts Information}
\description{
\code{lamhosts} finds the host name associated with its node number. Can be used 
by \code{\link{mpi.spawn.Rslaves}} to spawn R slaves on selected hosts. This is 
a LAM-MPI specific function.

\code{mpi.is.master} checks if it is running on master or slaves.

\code{mpi.hostinfo} finds an individual host information including rank and 
size in a comm.

\code{slave.hostinfo} is executed only by master and find all master and slaves 
host information in a comm.
}
\usage{
lamhosts()
mpi.is.master()
mpi.hostinfo(comm = 1)
slave.hostinfo(comm = 1) 
}
\arguments{
 \item{comm}{a communicator number}
}
\value{
  \code{lamhosts} returns CPUs nodes numbers with their host names.

  \code{mpi.is.master} returns TRUE if it is on master and FALSE otherwise.

  \code{mpi.hostinfo} sends to stdio a host name, rank, size and comm.

  \code{slave.hostname} sends to stdio a list of host, rank, size, and comm 
information for all master and slaves.
} 
\seealso{
  \code{\link{mpi.spawn.Rslaves}}
}
\author{
Hao Yu
}
\keyword{utilities}

\eof
\name{bin.nchar}
\alias{bin.nchar}
\alias{mpi.remote.fun}
\alias{mpi.wrap.fun}
\alias{getpid}
\alias{mpi.remote.slave}
\alias{mpi.parallel.slave}
\alias{string}
\alias{typeindex}
\alias{mpi.comm.is.null}
\title{Internal functions}
\description{
Internal functions used by other MPI functions.

\code{bin.nchar} is used to find the number of char in a binary 
character vector (length 1) produced by 
\code{\link[serialize]{serialize}}.

\code{getpid} is used to get R process id which then is used for setting R slave 
log files.

\code{mpi.comm.is.null} is used to test if a comm is MPI\_COMM\_NULL (empty 
members).

\code{mpi.remote.fun} is used by \code{\link{mpi.remote.exec}} for preparing a 
remote function with master variables.

\code{mpi.wrap.fun} is used by \code{\link{mpi.parallel.sim}} for preparing a 
wrap function.

\code{mpi.remote.slave} is executed on R slaves by 
\code{\link{mpi.remote.exec}} (to carry out a remote command). 

\code{mpi.parallel.slave} is executed on R slaves by 
\code{\link{mpi.parallel.sim}} (to carry out parallel Monto Carlo 
simulation). 

\code{string}  create a string (empty space character) buffer.

\code{typeindex} checks an R object mode (integer, double, char, or others).
}
\usage{
bin.nchar(x)
getpid()
mpi.comm.is.null(comm)
mpi.remote.fun(cmd, ..., needsub = TRUE, width.cutoff = 500)
mpi.wrap.fun(cmd,arg,cmd.name=substitute(cmd))
mpi.remote.slave(cmd, tag, ret)
mpi.parallel.slave()
string(length)
typeindex(x)
}
\arguments{
  \item{comm}{a communicator number.}
  \item{cmd}{a function name.}
  \item{arg}{optional argument list to \code{cmd}.}
  \item{cmd.name}{a function name as a string.}
  \item{...}{argument lists to be passed to cmd.}
  \item{needsub}{logical.}
  \item{width.cutoff}{an option used by \code{deparse}.}
  \item{tag}{a tag number}
  \item{ret}{logical.}
  \item{x}{an R object.}
  \item{length}{length of a string.}
}
\value{
\code{bin.nchar} returns the length of a binary char vector.

\code{getpid} returns an integer (process id).

\code{mpi.remote.fun} returns a string (a function with string quotation).

\code{mpi.wrap.fun} returns a function with argument values replaced from 
\code{arg}. If there is no ... argument in \code{cmd}, the original 
function is returned. Otherwise a wrap function is returned (it is a wrap 
to the function \code{cmd}.) 

\code{string} returns an empty character string.

\code{typeindex} returns \code{c(1, length(x))} if \code{x} is an integer 
vector; \code{c(2,length(x))} if \code{x} is a double vector; and -1 for other 
objects. } 
\seealso{
  \code{\link{mpi.spawn.Rslaves}}, 
  \code{\link{mpi.remote.exec}}.
  \code{\link{mpi.parallel.sim}}.
}
}
\author{
Hao Yu
}
\keyword{utilities}

\eof
\name{mpi.abort}
\title{MPI\_Abort API}
\usage{
  mpi.abort(comm = 1)
}
\alias{mpi.abort}
\arguments{
  \item{comm}{a communicator number}
}
\description{
  \code{mpi.abort} makes a ``best attempt" to abort all tasks in a comm.
}

\value{
  1 if success. Otherwise 0. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.finalize}}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.barrier}
\title{MPI\_Barrier API}
\usage{
  mpi.barrier(comm = 1)
}
\alias{mpi.barrier}
\arguments{
  \item{comm}{a communicator number}
}
\description{
  \code{mpi.barrier} blocks the caller until all members have called it.
}
\value{
  1 if success. Otherwise 0. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.bcast}
\title{MPI\_Bcast API}
\alias{mpi.bcast}
\description{
  \code{mpi.bcast} is a collective call among all members in a comm. It 
  broadcasts a message from the specified rank to all members. 
}
\usage{
mpi.bcast(x, type, rank = 0, comm = 1)
}
\arguments{
  \item{x}{data to be sent or received. Must be the same 
  type among all members.}
  \item{type}{1 for integer, 2 for double, and 3 for 
  character. Others are not supported.}
  \item{rank}{the sender.}
  \item{comm}{a communicator number.}
}
\value{
  \code{mpi.bcast} returns the message broadcasted by the sender 
  (specified by the rank).
} 
\details{
  \code{mpi.bcast} is a blocking call among all members in a comm, i.e, 
  all members have to wait until everyone calls it. All members have to 
  prepare the same type of messages (buffers). Hence it is relatively 
  difficult to use in R environment since the receivers may not know what 
  types of data to receive, not mention the length of data. Users should 
  use various extensions of \code{mpi.bcast} in R. They are 
  \code{\link{mpi.bcast.Robj}}, \code{\link{mpi.bcast.cmd}}, and 
\code{\link{mpi.bcast.Robj2slave}}.
 }
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
  \code{\link{mpi.bcast.Robj}},
  \code{\link{mpi.bcast.cmd}},
  \code{\link{mpi.bcast.Robj2slave}}.
}
%\examples{
%}
\keyword{interface}

\eof
\name{mpi.bcast.Robj}
\title{Extensions of MPI\_Bcast API}
\alias{mpi.bcast.Robj}
\alias{mpi.bcast.Robj2slave}
\description{
  \code{mpi.bcast.Robj} and \code{mpi.bcast.Robj2slave} are used to move  
  a general R object around among master and all slaves. 
}
\usage{
mpi.bcast.Robj(obj = NULL, rank = 0, comm = 1)
mpi.bcast.Robj2slave(obj, comm = 1)
}
\arguments{
  \item{obj}{an R object to be transmitted from the sender}
  \item{rank}{the sender.}
  \item{comm}{a communicator number.}
}
\value{
  \code{mpi.bcast.Robj} returns no value for the sender and the 
transmitted one for others. \linebreak  \code{mpi.bcast.Robj2slave} 
returns no value for the master 
and the transmitted R object along its name on slaves. 
} 
\details{
  \code{mpi.bcast.Robj} is an extension of \code{\link{mpi.bcast}} for 
  moving a general R object around from a sender to everyone. 
  \code{mpi.bcast.Robj2slave} does an R object transmission from 
  master to all slaves only. serialize package is used to 
serialize/unserialize an R object. 
}
\author{
Hao Yu
}
\seealso{
  \code{\link{mpi.send.Robj}},
  \code{\link{mpi.recv.Robj}},
}
%\examples{
%}
\keyword{utilities}

\eof
\name{mpi.bcast.cmd}
\title{Extension of MPI\_Bcast API}
\alias{mpi.bcast.cmd}
\description{
  \code{mpi.bcast.cmd} is an extension of \code{\link{mpi.bcast}}. 
  It is mainly used to transmit a command from master to all R slaves 
  spawned by using slavedaemon.R script.
}
\usage{
mpi.bcast.cmd(cmd=NULL, rank = 0, comm = 1)
}
\arguments{
  \item{cmd}{a command to be sent for master.}
  \item{rank}{the sender}
  \item{comm}{a communicator number}
}
\value{
  \code{mpi.bcast.cmd} returns no value for the sender and an expression 
  of the transmitted command for others.
} 
\details{
  \code{\link{deparse}} is used to convert a command into a char vector. 
  Some syntax errors of the command will be automatically detected by 
  \code{deparse}. Then the char vector is broadcasted to all members. After 
  receiving the char vector, the receivers use \code{\link{parse}} to convert 
  the char vector into an expression. Finally slavedaemon.R script will evaluate 
  this expression. Hence \code{mpi.bcast.cmd} is only used by slavedaemon.R, 
  or by whatever a R script based on slavedaemon.R. Please use  
 \code{\link{mpi.remote.exec}} if you want the executed results returned from R 
  slaves.
 }
\author{
Hao Yu
}
\seealso{
  \code{\link{mpi.remote.exec}}
}
%\examples{
%}
\keyword{utilities}

\eof
\name{mpi.comm.size}
\title{MPI\_Comm\_dup, MPI\_Comm\_rank. and MPI\_Comm\_size APIs}
\usage{
  mpi.comm.dup(comm, newcomm)
  mpi.comm.rank(comm = 1)
  mpi.comm.size(comm = 1)
}
\alias{mpi.comm.dup}
\alias{mpi.comm.rank}
\alias{mpi.comm.size}
\arguments{
  \item{comm}{a communicator number}
  \item{newcomm}{a new communicator number}
}
\description{
  \code{mpi.comm.dup} duplicates (copies) a comm to a 
new comm. \code{mpi.comm.rank} 
returns its rank in a comm. \code{mpi.comm.size} returns 
the total number of members in a comm. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\examples{
}
\keyword{interface}
 

\eof
\name{mpi.comm.disconnect}
\title{MPI\_Comm\_disconnect API}
\usage{
mpi.comm.disconnect(comm=1)
}
\alias{mpi.comm.disconnect}
\arguments{
  \item{comm}{a communicator number}
}
\description{
  \code{mpi.comm.disconnect} disconnects itself from a communicator and then 
  deallocates the communicator so it points to MPI\_COMM\_NULL.
}
\details{
  When members associated with a communicator finish jobs or exit, they have to 
  call \code{mpi.comm.disconnect} to release resource if the communicator was 
  created from an intercommunicator by \code{\link{mpi.intercomm.merge}}. If 
  \code{\link{mpi.comm.free}} is used instead, \code{\link{mpi.finalize}} called 
  by slaves may cause undefined impacts on master who wishes to stay.
}
\value{
  1 if success. Otherwise 0. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.comm.free}}
}
%\examples{
%}
\keyword{interface}

\eof
\name{mpi.comm.free}
\title{MPI\_Comm\_free API}
\usage{
  mpi.comm.free(comm=1)
}
\alias{mpi.comm.free}
\arguments{
  \item{comm}{a communicator number}
}
\description{
  \code{mpi.comm.free}  deallocates a communicator so it 
  points to MPI\_COMM\_NULL.
}
\details{
  When members associated with a communicator finish jobs or exit, they have to 
  call \code{mpi.comm.free} to release resource so \code{\link{mpi.comm.size}} 
  will return 0. If the comm  was created from an intercommunicator by 
  \code{\link{mpi.intercomm.merge}}, use \code{\link{mpi.comm.disconnect}} instead.
}

\value{
  1 if success. Otherwise 0. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.comm.disconnect}}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.comm.get.parent}
\title{MPI\_Comm\_get\_parent, MPI\_Comm\_remote\_size, MPI\_Comm\_test\_inter 
APIs}
\usage{
  mpi.comm.get.parent(comm = 2)
  mpi.comm.remote.size(comm = 2)
  mpi.comm.test.inter(comm = 2)
}
\alias{mpi.comm.get.parent}
\alias{mpi.comm.remote.size}
\alias{mpi.comm.test.inter}
\arguments{
  \item{comm}{an intercommunicator number.}
}
\description{
  \code{mpi.comm.get.parent} is mainly used by slaves to find the 
intercommunicator or the parent who spawns them. The intercommunicator is saved 
in the specified comm number.

\code{mpi.comm.remote.size} is mainly used by master to find the total number of 
slaves spawned.

\code{mpi.comm.test.inter} tests if a comm is an intercomm or not.
}

\value{
\code{mpi.comm.get.parent} and \code{mpi.comm.test.inter} return  1 if success 
and 0 otherwise. 

\code{mpi.comm.remote.size} returns the total number of members in the remote 
group in an intercomm. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.intercomm.merge}}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.comm.set.errhandler}
\title{MPI\_Comm\_set\_errhandler API}
\usage{
mpi.comm.set.errhandler(comm = 1)
}
\alias{mpi.comm.set.errhandler}
\arguments{
  \item{comm}{a communicator number}
}
\description{
  \code{mpi.comm.set.errhandler} sets a communicator to MPI\_ERRORS\_RETURN 
instead of  
MPI\_ERRORS\_ARE\_FATAL (default) which crashes R on any type of MPI errors.  
Almost all MPI API calls return errcodes which can map to specific MPI error 
messages. All MPI related error messages come from predefined 
MPI\_Error\_string. }
\value{
  1 if success. Otherwise 0. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.comm.spawn}
\alias{mpi.comm.spawn}
\title{MPI\_Comm\_spawn API }
\description{
\code{mpi.comm.spawn} tries to start \code{nslaves} identical copies of 
\code{slaves}, establishing communication with them and returning an 
intercommunicator. The spawned slaves are referred to as children, and the 
process that spawned them is called the parent (master). The children have 
their own MPI\_COMM\_WORLD represented by comm 0. To make communication 
possible among master and slaves, all slaves should use 
\code{\link{mpi.comm.get.parent}} to find their parent and use 
\code{\link{mpi.intercomm.merge}} to merger an intercomm to a comm.
 }
\usage{
 mpi.comm.spawn(slave, slavearg = character(0),
                nslaves = mpi.universe.size(), info = 0,
                root = 0, intercomm = 2)
}
\arguments{
  \item{slave}{a file name to an executable program.}
  \item{slavearg}{an argument list (a char vector) to slave.}
  \item{nslaves}{number of slaves to be spawned.}
  \item{info}{an info number.}
  \item{root}{the root member who spawns slaves.}
  \item{intercomm}{an intercomm number.}
}
\value{
A message is printed to indicate how many slaves are successfully 
spawned and how many failed.
}
\seealso{
  \code{\link{mpi.comm.get.parent}},
  \code{\link{mpi.intercomm.merge}}.
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}

\author{
Hao Yu
}
\keyword{interface}

\eof
\name{mpi.any.source}
\alias{mpi.any.source}
\alias{mpi.any.tag}
\alias{mpi.proc.null}
\title{MPI Constants}
\description{
  Find MPI constants: MPI\_ANY\_SOURCE, MPI\_ANY\_TAG, or MPI\_PROC\_NULL
}
\usage{
mpi.any.source()
mpi.any.tag()
mpi.proc.null()
}
\arguments{
  None
}
\value{
  Each function returns an integer value.
}
\details{
  These constants are mainly used by 
\code{\link{mpi.send}}, \code{\link{mpi.recv}}, and 
\code{\link{mpi.probe}}. 
Different implementation of MPI may use different 
  integers for MPI\_ANY\_SOURCE, MPI\_ANY\_TAG, and MPI\_PROC\_NULL. Hence one 
  should use these functions instead  real integers for MPI communications.
} 
\seealso{
  \code{\link{mpi.send}}, \code{\link{mpi.recv}}.
}
\references{
  \url{http://www.lam-mpi.org}, \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\keyword{utilities}

\eof
\name{mpi.exit}
\alias{mpi.exit}
\alias{mpi.quit}
\title{Exit MPI Environment }
\description{
\code{mpi.exit} terminates MPI execution environment and detaches the 
library Rmpi. After that, you can still work on R.

\code{mpi.quit} terminates MPI execution environment and quits R.
}
\usage{
mpi.exit()
mpi.quit(save = "no")
}
\arguments{
\item{save}{the same argument as \code{quit} but default to "no".}
}
\value{
 \code{mpi.exit} always returns 1
}
\details{
  Normally, \code{\link{mpi.finalize}} is used to clean all MPI states. 
  However, it will not detach the library Rmpi. To be more safe leaving MPI,  
  \code{mpi.exit} not only calls \code{mpi.finalize} but also detaches the 
  library Rmpi. This will make reload the library Rmpi impossible.

 If leaving MPI and R altogether, one simply uses \code{mpi.quit}. 
} 
\seealso{
  \code{\link{mpi.finalize}}
}
\author{
Hao Yu
}
\keyword{utilities}

\eof
\name{mpi.finalize}
\alias{mpi.finalize}
\title{MPI\_Finalize API}
\description{
  Terminates MPI execution environment. 
}
\usage{
  mpi.finalize()
}
\arguments{
  None
}
\value{
  Always return 1
}
\details{
  This routines must be called by each slave (master) before it exists. This 
  call cleans all MPI state. Once \code{mpi.finalize} has been called, no MPI 
  routine may be called. To be more safe leaving MPI, please use 
  \code{\link{mpi.exit}} which not only calls \code{mpi.finalize} but also 
  detaches the library Rmpi. This will make reload the library Rmpi impossible. 
} 
\author{
Hao Yu
}
\seealso{
  \code{\link{mpi.exit}}
}
\references{
  \url{http://www.lam-mpi.org/},
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
%\examples{ 
%}
\keyword{interface}

\eof
\name{mpi.gather}
\title{MPI\_Gather, MPI\_Gatherv, MPI\_Allgather, and MPI\_Allgatherv APIs}
\alias{mpi.gather}
\alias{mpi.gatherv}
\alias{mpi.allgather}
\alias{mpi.allgatherv}
\description{
  \code{mpi.gather} and \code{mpi.gatherv} (vector variant) gather each 
  member's message to the member specified by the argument \code{root}.
  The root member receives the messages and stores them in rank 
  order. \code{mpi.allgather} and \code{mpi.allgatherv} are the same as
  \code{mpi.gather} and \code{mpi.gatherv} except that all members receive
  the result instead of just the root.
}
\usage{
mpi.gather(x, type, rdata, root = 0, comm = 1) 
mpi.gatherv(x, type, rdata, rcounts, root = 0, comm = 1) 

mpi.allgather(x, type, rdata, comm = 1) 
mpi.allgatherv(x, type, rdata, rcounts, comm = 1) 
}
\arguments{
  \item{x}{data to be gathered. Must be the same type.}
  \item{type}{1 for integer and 2 for double. Others are not supported.}
  \item{rdata}{the receive buffer. Must be the same type as the sender and big 
  enough to include all message gathered.}
  \item{rcounts}{int vector specifying the length of each message.}
  \item{root}{rank of the receiver}
  \item{comm}{a communicator number}
}
\value{
  For \code{mpi.gather} or \code{mpi.gatherv}, it returns the gathered 
  message for the root member. For other members, it returns what is in rdata, 
  i.e., rdata (or rcounts) is ignored. For \code{mpi.allgather} or 
  \code{mpi.allgatherv}, it returns the gathered message for all members. 
} 
\details{
  For \code{mpi.gather} and \code{mpi.allgather}, the message to be gathered 
must be the same dim and the same type. The receive buffer can be prepared as 
either integer(size * dim) or double(size * dim), where size is the total 
number of members in a comm. For \code{mpi.gatherv} and \code{mpi.allgatherv}, 
the message to be gathered can have different dims but must be the same type. 
The argument \code{rcounts} records these different dims into an integer vector 
in rank order. Then the receive buffer can be prepared as either 
integer(sum(rcounts)) or double(sum(rcounts)).  
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/},
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
  \code{\link{mpi.scatter}}, \code{\link{mpi.scatterv}}.
}
%\examples{
%}
\keyword{interface}

\eof
\name{mpi.get.count}
\title{MPI\_Get\_count API}
\usage{
mpi.get.count(type, status = 0)
}
\alias{mpi.get.count}
\arguments{
  \item{type}{1 for integer, 2 for double, 3 for char.}
  \item{status}{a status number}
}
\description{
  \code{mpi.get.count} finds the length of a received message.
}
\details{
  When \code{\link{mpi.recv}} is used to receive a message, the receiver 
  buffer can be set to be bigger than the incoming message. To find the 
  exact length of the received message, \code{mpi.get.count} is used to 
  find its exact length. \code{mpi.get.count} must be called 
  immediately after calling \code{mpi.recv} otherwise the status may be 
  changed.
}
\value{
  the length of a received message.
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.send}}, \code{\link{mpi.recv}},
 \code{\link{mpi.get.sourcetag}}, \code{\link{mpi.probe}}.
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.get.processor.name}
\title{MPI\_Get\_processor\_name API}
\usage{
  mpi.get.processor.name(short = TRUE)
}
\alias{mpi.get.processor.name}
\arguments{
 \item{short}{a logical.}
}
\description{
  \code{mpi.get.processor.name} returns the host name (a string) where 
it is executed. 
}
\value{
a base host name if \code{short = TRUE} and a full host name otherwise.
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.get.sourcetag}
\title{Utility for finding the source and tag of a received message}
\usage{
  mpi.get.sourcetag(status = 0)
}
\alias{mpi.get.sourcetag}
\arguments{
  \item{status}{a status number}
}
\description{
  \code{mpi.get.sourcetag} finds the source and tag of a received message.
}
\details{
  When \code{\link{mpi.any.source}} and/or \code{\link{mpi.any.tag}} are 
  used by \code{\link{mpi.recv}} or \code{\link{mpi.probe}}, one can use 
\code{mpi.get.sourcetag} 
  to find who sends the message or with what a tag number. 
  \code{\link{mpi.get.sourcetag}} must be called immediately after calling 
  \code{mpi.recv} or \code{mpi.probe} otherwise the obtained information may not 
be right.
}
\value{
  2 dim int vector. The first integer is the source and the second is the 
tag.
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.send}}, \code{\link{mpi.recv}}, \code{\link{mpi.probe}},
 \code{\link{mpi.get.count}}
}
%\examples{
%}
\keyword{utilities}
 

\eof
\name{mpi.info.create}
\alias{mpi.info.create}
\alias{mpi.info.free}
\alias{mpi.info.get}
\alias{mpi.info.set}
\title{MPI\_Info\_create, MPI\_Info\_free, MPI\_Info\_get, MPI\_Info\_set APIs}
\description{
Many MPI APIs take an info argument for additional information passing. An info 
is an object which consists of many (key,value) pairs. Rmpi uses an internal 
memory to store an info object.

\code{mpi.info.create} creates a new info object.

\code{mpi.info.free} frees an info object and sets it to MPI\_INFO\_NULL.

\code{mpi.info.get}  retrieves the value associated with key in an info.

\code{mpi.info.set} adds the key and value pair to info.
}
\usage{
  mpi.info.create(info = 0)
  mpi.info.free(info = 0)
  mpi.info.get(info = 0, key, valuelen)
  mpi.info.set(info = 0, key, value)
}
\arguments{
  \item{info}{an info number.}
  \item{key}{a char (length 1).}
  \item{valuelen}{the length (nchar) of a key}
  \item{value}{a char (length 1).}
}
\value{
  \code{mpi.info.create}, \code{mpi.info.free}, and \code{mpi.info.set}  return 
1 if success and 0 otherwise.

\code{mpi.info.get} returns the value (a char) for a given info and valuelen.
}
\seealso{
  \code{\link{mpi.spawn.Rslaves}}
}
\author{
Hao Yu
}
\keyword{interface}

\eof
\name{mpi.init.sprng}
\alias{mpi.init.sprng}
\title{Interface to rsprng for initializing SPRNG}
\description{
  \code{mpi.init.sprng} is used to initialize the parallel random 
   number generator (PRNG) states in MPI environment. It is a collective 
   call so all members in a comm must call it together.
}
\usage{
mpi.init.sprng(seed = floor(runif(1, 1, 2147483648)), 
		   kindprng = "default",
                   para = 0,
		   rank = 0,
		   comm = 1)
}
\arguments{
  \item{seed}{an integer of random number seed.} 
  \item{kindprng}{a character string of the desired kind of parallel
    random number generator.}
  \item{para}{additional parameters for the parallel random number
    generators.  If \code{para} is 0, default parameters for each PRNG
    are used.  When invalid parameter is given, a warning is issued and
    the default parameter is used.}
  \item{rank}{the sender who distributes seed, kindprng, para to other 
   members in a comm, i.e., other members' seed, kindprng, and para are 
   ignored.}
  \item{comm}{a comm number.}
}
\value{
  Return the a two-element character vector of the RNG and normal kinds
  in use before the call.
}
\section{Warning}{
  When \code{sprng} is not used any more, please run 
  \code{\link[rsprng]{free.sprng}} on all slaves and master to set back 
  to R default RNG. If not doing so, an error message will come out like 
  ".Random.seed[1] = 5 but no user-supplied generator" in the next 
  session. One quick remedy is \code{rm(.Random.seed)}. 
}
\references{
  SPRNG: Scalable Parallel Random Number Generator Library Web Page.
  \url{http://sprng.cs.fsu.edu/}
}
\seealso{
  \code{\link[rsprng]{init.sprng}}
}
\author{
  Hao Yu
}
\keyword{interface}
\keyword{distribution}

\eof
\name{mpi.intercomm.merge}
\title{MPI\_Intercomm\_merge API}
\usage{
mpi.intercomm.merge(intercomm=2, high=0, comm=1)
}
\alias{mpi.intercomm.merge}
\arguments{
  \item{intercomm}{an intercommunicator number}
  \item{high}{Used to order the groups of the two intracommunicators within comm 
when creating the new communicator}
  \item{comm}{a (intra)communicator number}
}
\description{
  Creates an intracommunicator from an intercommunicator 
}
\details{
  When master spawns slaves, an intercommunicator is created. To make 
  communications (point-to-point or groupwise) among master and slaves, an 
  intracommunicator must be created. \code{mpi.intercomm.merge} is used for 
  that purpose. This is a collective call so all master and slaves call 
  together. R slaves spawned by \code{\link{mpi.spawn.Rslaves}} should use
\code{\link{mpi.comm.get.parent}} to get (set) an intercomm to a number followed 
by merging antercomm to an intracomm. One 
can use \code{\link{mpi.comm.test.inter}} to test if a 
  communicator is an intercommunicator or not.
}
\value{
  1 if success. Otherwise 0. 
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org}, \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.comm.test.inter}}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.parallel.sim}
\alias{mpi.parallel.sim}
\title{Parallel Monto Carlo Simulation}
\description{
 Carry out parallel Monto Carlo simulation on R slaves spawned by 
 using slavedaemon.R script and all executed results are returned back to 
 master. 
} 
\usage{
mpi.parallel.sim(n=100, rand.gen=rnorm, rand.arg=NULL,statistic, 
nsim=100, run=1, slaveinfo=TRUE, comm=1, ...)  
}
\arguments{
  \item{n}{sample size.}
  \item{rand.gen}{the random data generating function. See the details 
section}
  \item{rand.arg}{additional argument list to \code{rand.gen}.}
  \item{statistic}{the statistic function to be simulated. See the 
details section}
  \item{nsim}{the number of simulation carried on a slave which is 
 counted as one slave job.}
  \item{run}{the number of looping. See the details section.}
  \item{slaveinfo}{if TRUE, the numbers of jobs finished by slaves 
will be displayed.}
  \item{comm}{a communicator number}
  \item{...}{optional arguments to \code{statistic}}
}
\value{
   The returned values depend on values returned by \code{\link{sapply}} 
after applying \code{statistic} and the total number of simulation 
(TNS). If \code{statistic} returns a single value, then the
result is a vector of length TNS. If \code{statistic} returns a vector 
(list) of length \code{ncol}, then the result is a matrix of dimension 
\code{c(TNS,ncol)}.   
}
\details{
It is assumed that one simulation is carried out as 
\code{statistic(rand.gen(n))}, where \code{rand.gen(n)} can return any 
values as long as \code{statistic} can take them. Additional arguments can 
be passed to \code{rand.gen} by \code{rand.arg} as a list. Optional 
arguments can also be passed to \code{statistic} by the argument 
\code{...}. 

To pass values from \code{rand.arg} and/or \code{...} as default values, 
\code{\link{mpi.wrap.fun}} is used. If \code{rand.gen} (\code{statistic}) 
does not contain the argument \code{...}, \code{\link{mpi.wrap.fun}} 
returns the same function with default values replaced. Otherwise it 
returns a wrap function so that additional arguments can be passed 
properly. It is recommended to use \code{rand.gen} and \code{statistic} 
without \code{...} argument.

Each slave job consists of 
\code{sapply(rep(n,nsim),function(n)statistic(rand.gen(n)))}, 
i.e., each job runs \code{nsim} number of simulation. The returned values 
are transported from slaves to master.

The total number of simulation (TNS) is calculated as follows. Let 
slave.num be the total number of slaves in a \code{comm} and it is 
\code{mpi.comm.size(comm)-1}. Then TNS=slave.num*nsim*run and the total
number of slave jobs is slave.num*run, where \code{run} is the number of 
looping from master perspective. If run=1, each slave will run one slave 
job. If run=2, each slave will run two slaves jobs on average, and so on. 

The purpose of using \code{run} has two folds. It allows a tuneup 
of slave job size and total number of slave jobs to deal with two 
different cluster environments. On a cluster of slaves with equal CPU 
power, \code{run=1} is often enough. But if \code{nsim} is too big, one
can set \code{run=2} and the slave jog size to be \code{nsim/2} so that  
TNS=slave.num*(nsim/2)*(2*run). This may improve R computation 
efficiency slightly. On a cluster of slaves with different CPU power, one 
can choose a big value of \code{run} and a small of value of \code{nsim} 
so that master can dispatch more jobs to slaves who run faster than 
others. This will keep all slaves busy so that load balancing is 
achieved. 

See the warning section before you use \code{mpi.parallel.sim}.
}

\section{Warning}{
It is assumed that the package \pkg{rsprng} must be installed and SPRNG 
must be properly initialized. \code{\link{mpi.spawn.Rslaves}} will 
initialize SPRNG as default. Though \code{mpi.parallel.sim} is still 
functional without SPRNG, the quality of simulation is not guarantied. 
See \code{\link{mpi.init.sprng}} how to initialize SPRNG.

\code{mpi.parallel.sim} will automatically transfer \code{rand.gen} 
and \code{statistic} to slaves. However, any functions that 
\code{rand.gen} and \code{statistic} reply on but are no in slaves 
must be transfered to slaves before using \code{mpi.parallel.sim}. You 
can use \code{\link{mpi.bcast.Robj2slave}} for that purpose. The same is 
applied to required packages or C/Fortran codes. You can use either 
\code{\link{mpi.bcast.cmd}} or put \code{required(package)} and/or 
\code{dyn.load(so.lib)} into \code{rand.gen} and \code{statistic}.
 
It is recommended that \code{statistic} should return a numeric vector or 
a list so master can use \code{c} to combine values together as a vector 
or a matrix. Master may have difficult to combine other values. 
} 

\seealso{
  \code{\link{mpi.spawn.Rslaves}},
  \code{\link{mpi.bcast.cmd}}
  \code{\link{mpi.bcast.Robj2slave}}
}
\author{
Hao Yu
}
\keyword{utilities}

\eof
\name{mpi.probe}
\title{MPI\_Probe API}
\usage{
mpi.probe(source, tag, comm = 1, status = 0)
}
\alias{mpi.probe}
\arguments{
  \item{source}{the source of incoming message or mpi.any.source() for any 
source.}
  \item{tag}{a tag number or mpi.any.tag() for any tag.}
  \item{comm}{a communicator number}
  \item{status}{a status number}
}
\description{
  \code{mpi.probe} uses the source and tag of incoming message to set a status.
}
\details{
  When \code{\link{mpi.send}} is used to send a message, the receiver may not 
  know the exact length before receiving it. \code{mpi.probe} is used to probe 
  the incoming message and put some information into a status. Then the exact 
  length can be found by using \code{\link{mpi.get.count}} to such a status.
If the wild card \code{mpi.any.source} or \code{mpi.any.tag} are used, then one 
can use \code{\link{mpi.get.sourcetag}} to find the exact source or tag of a 
sender. }
\value{
  1 if success; 0 otherwise.
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
 \code{\link{mpi.send}}, \code{\link{mpi.recv}},
 \code{\link{mpi.get.count}}
}
%\examples{
%}
\keyword{interface}
 

\eof
\name{mpi.reduce}
\title{MPI\_Reduce and MPI\_Allreduce APIs}
\alias{mpi.allreduce}
\alias{mpi.reduce}
\description{
  \code{mpi.reduce} and \code{mpi.allreduce} are global reduction operations. 
  \code{mpi.reduce} combines each member's result, using the operation 
  \code{op}, and returns the combined value(s) to the member specified by 
  the argument \code{dest}. \code{mpi.allreduce} is the same as 
  \code{mpi.reduce} except that all members receive the combined value(s).
}
\usage{
mpi.reduce(x, type=2, op=c("sum","prod","max","min","maxloc","minloc"), 
	dest = 0, comm = 1) 

mpi.allreduce(x, type=2, op=c("sum","prod","max","min","maxloc","minloc"), 
	comm = 1) 
}
\arguments{
  \item{x}{data to be reduced. Must be the same dim and the same type for all 
  members.}
  \item{type}{1 for integer and 2 for double. Others are not supported.}
  \item{op}{one of "sum", "prod", "max", "min", "maxloc", or "minloc".} 
  \item{dest}{rank of destination}
  \item{comm}{a communicator number}
}
\value{
  \code{mpi.reduce} returns the combined value(s) to the member specified  
  by \code{dest}. \code{mpi.allreduce} returns the combined values(s) to 
  every member in a comm. The combined value(s) may be the summation, 
  production, maximum, or minimum specified by the argument \code{op}. If 
  the \code{op} is either "maxloc" or "minloc", then the maximum (minimum) 
  value(s) along the maximum (minimum) rank(s) will be returned.
}
\details{
  It is important that all members in a comm call either all \code{mpi.reduce} 
  or all \code{mpi.allreduce} even though the master may not be in 
  computation. They must provide exactly the same type and dim 
  vectors to be reduced. If the operation "maxloc" or "minloc" is used, 
  the combined vector is twice as long as the original one since the 
  maximum or minimum ranks are included. 
} 
\author{
  Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/},  
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
  \code{\link{mpi.gather}}.
}
%\examples{
%}
\keyword{interface}

\eof
\name{mpi.remote.exec}
\alias{mpi.remote.exec}
\title{Remote Executions on R slaves}
\description{
 Remotely execute a command on R slaves spawned by using slavedaemon.R script 
 and return all executed results back to master. 
} 
\usage{
mpi.remote.exec(cmd, ..., comm = 1, ret = TRUE)  
}
\arguments{
  \item{cmd}{the command to be executed on R slaves}
  \item{...}{used as arguments to cmd (function command) for passing their 
  (master) values to R slaves, i.e., if `myfun(x)' will be executed on R slaves 
with `x' as master variable, use mpi.remote.exec(cmd=myfun, x).}
  \item{comm}{a communicator number.}
  \item{ret}{return executed results from R slaves if TRUE.}
}
\value{
  return executed results from R slaves if the argument \code{ret} is 
  set to be TRUE. The value could be a data.frame if values 
  (integer or double) from each slave have the same dimension. 
  Otherwise a list is returned.}
\details{
  Once R slaves are spawned by \code{\link{mpi.spawn.Rslaves}} with the 
  slavedaemon.R script, they are waiting for instructions from master. One can 
  use \code{\link{mpi.bcast.cmd}} to send a command to R slaves. However it 
  will not return executed results. Hence \code{mpi.remote.exec} can be 
  considered an extension to \code{\link{mpi.bcast.cmd}}.    
}

\section{Warning}{
  \code{mpi.remote.exec} may have difficult guessing invisible executed results 
  on R slaves. Use `ret = FALSE' instead. Please do not pass large amount of 
  data through \code{...} to a function command. The use of the argument 
  \code{...} is just for the sake of passing some simple variables from master 
  to R slave.
} 
\seealso{
  \code{\link{mpi.spawn.Rslaves}},
  \code{\link{mpi.bcast.cmd}}
}
\author{
Hao Yu
}
\keyword{utilities}

\eof
\name{mpi.scatter}
\title{MPI\_Scatter and MPI\_Scatterv APIs}
\alias{mpi.scatter}
\alias{mpi.scatterv}
\description{
  \code{mpi.scatter} and \code{mpi.scatterv} are the inverse operations of 
  \code{\link{mpi.gather}} and \code{\link{mpi.gatherv}} respectively.
}
\usage{
mpi.scatter(x, type, rdata, root = 0,  comm = 1) 
mpi.scatterv(x, scounts, type, rdata, root = 0, comm = 1)
}
\arguments{
  \item{x}{data to be scattered.}
  \item{type}{1 for integer and 2 for double. Others are not supported.}
  \item{rdata}{the receive buffer. Must be the same type as the sender} 
  \item{scounts}{int vector specifying the block length inside a message to be 
  scattered to other members.}
  \item{root}{rank of the receiver}
  \item{comm}{a communicator number}
}
\details{
  \code{mpi.scatter} scatters the message x to all members. Each member receives 
  a portion of x with dim as length(x)/size in rank order, where size is the 
  total number of members in a comm. So the receive buffer can be prepared as 
  either integer(length(x)/size) or double(length(x)/size). For 
\code{mpi.scatterv}, scounts counts the portions (different dims) of x sent to 
each member. Each member needs to prepare the receive buffer as either 
integer(scounts[i]) or double(scounts[i]). 
}
\value{
For non-root members, \code{mpi.scatter} or \code{scatterv} returns the
scattered message and ignores whatever is in x (or scounts). For the root 
  member, it returns the portion belonging to itself.
} 
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/},
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
  \code{\link{mpi.gather}}, \code{\link{mpi.gatherv}}.
}
%\examples{
%}
\keyword{interface}

\eof
\name{mpi.send}
\title{MPI\_Send and MPI\_Recv APIs}
\alias{mpi.send}
\alias{mpi.recv}
\description{
  The pair \code{mpi.send} and \code{mpi.recv} are two most used blocking 
  calls for point-to-point communications. An int, double or char vector 
  can be transmitted from any source to any destination.  
}
\usage{
mpi.send(x, type, dest, tag,  comm = 1)
mpi.recv(x, type, source, tag,  comm = 1, status = 0)
}
\arguments{
  \item{x}{data to be sent or received. Must be the same 
  type for source and destination. The receive buffer must be as large as 
  the send buffer.}
  \item{type}{1 for integer, 2 for double, and 3 for 
  character. Others are not supported.}
  \item{dest}{the destination rank. Use \code{mpi.proc.null} for a 
  fake destination.}
  \item{source}{the source rank. Use \code{mpi.any.source} for any source. 
   Use \code{mpi.proc.null} for a fake source. }
  \item{tag}{non-negative integer. Use \code{mpi.any.tag} for any tag flag.}
  \item{comm}{a communicator number.}
  \item{status}{a status number.}
}
\value{
  \code{mpi.send} returns no value. \code{mpi.recv} returns the int, 
  double or char vector sent from \code{source}. 
} 
\details{
  The pair \code{mpi.send} and \code{mpi.recv} must be used together, 
  i.e., if there is a sender, then there must be a receiver. Any mismatch 
  will result a deadlock situation, i.e., programs stop responding. The 
  receive buffer must be large enough to contain an incoming message 
  otherwise programs will be crashed. One can use \code{\link{mpi.probe}} and
  \code{\link{mpi.get.count}} to find the length of an incoming message 
  before calling \code{mpi.recv}. If \code{\link{mpi.any.source}} or 
  \code{\link{mpi.any.tag}} is used in \code{mpi.recv}, one can use 
  \code{\link{mpi.get.sourcetag}} to find out the source or tag of the 
  received message. To send/receive an R object rather than an int, double 
  or char vector, please use the pair \code{\link{mpi.send.Robj}} and  
  \code{\link{mpi.recv.Robj}}.
 }
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
  \code{\link{mpi.send.Robj}},
  \code{\link{mpi.recv.Robj}},
  \code{\link{mpi.probe}}.  
  \code{\link{mpi.get.count}}, 
  \code{\link{mpi.get.sourcetag}}. 
} 
%\examples{ 
%} 
\keyword{interface}


\eof
\name{mpi.send.Robj}
\title{Extensions of MPI\_Send and MPI\_Recv APIs}
\alias{mpi.send.Robj}
\alias{mpi.recv.Robj}
\description{
  \code{mpi.send.Robj} and \code{mpi.recv.Robj} are two 
  extensions of \code{mpi.send} and \code{mpi.recv}. They are used to 
  transmit a general R object from any source to any destination.  
}
\usage{
mpi.send.Robj(obj, dest, tag, comm = 1)
mpi.recv.Robj(source, tag, comm = 1, status = 0)
}
\arguments{
  \item{obj}{an R object. Can be any R object. }
  \item{dest}{the destination rank.}
  \item{source}{the source rank or mpi.any.source() for any source.} 
  \item{tag}{non-negative integer or mpi.any.tag() for any tag.}
  \item{comm}{a communicator number.}
  \item{status}{a status number.}
}
\value{
  \code{mpi.send.Robj} returns no value. \code{mpi.recv.Robj} returns the 
  the transmitted R object. 
} 
\details{
   \code{mpi.send.Robj} uses \code{\link[serialize]{serialize}} to encode 
an R object into 
   a binary char vector. It sends the message to the destination. The receiver 
decode the message back into an R object by using 
\code{\link[serialize]{unserialize}}. 
 }
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
\seealso{
  \code{\link{mpi.send}},
  \code{\link{mpi.recv}},
  \code{\link[serialize]{serialize}},
  \code{\link[serialize]{unserialize}}, 
}
%\examples{
%}
\keyword{utilities}


\eof
\name{mpi.spawn.Rslaves}
\alias{mpi.spawn.Rslaves}
\alias{mpi.close.Rslaves}
\alias{tailslave.log}
\title{Spawn and Close R Slaves}
\description{
\code{mpi.spawn.Rslaves} spawns R slaves to those hosts automatically 
chosen by MPI or specific hosts assigned by the argument \code{hosts}.
Those R slaves are running in R BATCH mode with a specific Rscript file. 
The default Rscript file "slavedaemon.R" provides interactive R slave 
environments.

\code{mpi.close.Rslaves} shuts down R slaves spawned by 
\code{mpi.spawn.Rslaves}.

\code{tailslave.log} view (from tail) R slave log files (assuming they are all 
in one working directory).
}
\usage{
mpi.spawn.Rslaves(Rscript=system.file("slavedaemon.R", package="Rmpi"),
        nslaves=mpi.universe.size(), root = 0, intercomm = 2,
        comm = 1, hosts = NULL, needlog = TRUE, needsprng = TRUE))

mpi.close.Rslaves(dellog = TRUE, comm = 1)
tailslave.log(nlines = 3, comm = 1)
}
\arguments{
  \item{Rscript}{an R script file used to run R in BATCH mode.}
  \item{nslaves}{number of slaves to be spawned.}
  \item{root}{the rank number of the member who spawns R slaves.}
  \item{intercomm}{an intercommunicator number}
  \item{comm}{a communicator number merged from an intercomm.}
  \item{hosts}{NULL or LAM node numbers to specify where R slaves to be 
spawned.}
  \item{needlog}{a logical. If TRUE, R BATCH outputs will be saved in log files.  
  If FALSE, the outputs will send to /dev/null.}
  \item{needsprng}{a logical. If TRUE and the package \pkg{rsprng} is 
installed on both master and slaves, SPRNG will be initialized.}  
  \item{dellog}{a logical specifying if R slave's log files are deleted or 
not.}
  \item{nlines}{number of lines to view from tail in R slave's log files.}
}
\value{
  \code{mpi.spawn.Rslaves} prints to stdio  how many 
slaves are successfully spawned and where they are running.

 \code{mpi.close.Rslaves} return 1 if success and 0 otherwise.

\code{tailslave.log} returns last lines of R slave's log files.
}
\details{
The R slaves that \code{mpi.spawn.Rslaves} spawns are really running a shell 
program which can be found in \code{system.file("Rslaves.sh",package="Rmpi")} 
which takes a Rscript 
file as one of its arguments. Other arguments are used to see if a log 
file (R output) is needed and how to name it. The master process id and 
the comm number, along with host names where R slaves are running  are 
used to name these log files. 

A warning message will appear if one tries to spawn more R slaves than 
total CPUs available in a cluster. 

Once R slaves are successfully spawned, the 
mergers from an intercomm (default `intercomm = 2') to a comm (default 
`comm = 1') are automatically done on master and slaves (should be done 
if the default Rscript is replaced). If additional sets of R slaves are 
needed, please use `comm = 3', `comm = 4', etc to spawn them. At most a 
comm number up to 10 can be used. Notice that the default comm number for 
R slaves (using slavedaemon.R) is always 1 which is saved as .comm.  


To spawn R slaves to specific hosts, please use the argument \code{hosts} 
with a list of those node numbers (an integer vector). Total node numbers 
along their host names can be found by using \code{\link{lamhosts}}.
Notice that this is LAM-MPI specific. 
} 

\seealso{
  \code{\link{mpi.comm.spawn}},
  \code{\link{lamhosts}}.
}
\author{
Hao Yu
}
\keyword{utilities}

\eof
\name{mpi.universe.size}
\title{MPI\_Universe\_size API}
\usage{
  mpi.universe.size()
}
\alias{mpi.universe.size}
\arguments{
  None.
}
\description{
  \code{mpi.universe.size} returns the total number of CPUs available in a 
cluster. Some MPI implements may not have this MPI call available.
}
\author{
Hao Yu
}
\references{
  \url{http://www.lam-mpi.org/}, 
  \url{http://www-unix.mcs.anl.gov/mpi/www/www3/}
}
%\examples{
%}
\keyword{interface}
 

\eof
